@@ -1,6 +1,6 @@
 /*
     Copyright (c)  2006, 2007		Dmitry Butskoy
-					<buc@citadel.stu.neva.ru>
+					<dmitry@butskoy.name>
     License:  GPL v2 or any later
 
     See COPYING for the status of this software.
@@ -19,7 +19,7 @@
 
 
 #define DEF_RADB_SERVER		"whois.radb.net"
-#define DEF_RADB_SERVICE	"whois"
+#define DEF_RADB_SERVICE	"nicname"
 
 
 static sockaddr_any ra_addr = {{ 0, }, };
