@@ -1,6 +1,6 @@
 /*
     Copyright (c)  2006, 2007		Dmitry Butskoy
-					<buc@citadel.stu.neva.ru>
+					<dmitry@butskoy.name>
     License:  GPL v2 or any later
 
     See COPYING for the status of this software.
@@ -169,6 +169,20 @@ void error_or_perm (const char *str) {
 }
 
 
+void put_err (probe *pb, const char *format, ...) {
+	va_list ap;
+	char *curr = pb->err_str;
+	char *end = pb->err_str + sizeof (pb->err_str) - 1;
+
+	/*  It can already contain something when `--mtu' or `-T -O mss'   */
+	while (curr < end && *curr)  curr++;
+
+	va_start (ap, format);
+	vsnprintf (curr, end - curr, format, ap);
+	va_end (ap);
+}
+
+
 /*  Set initial parameters according to how we was called   */
 
 static void check_progname (const char *name) {
@@ -259,7 +273,7 @@ static void make_fd_used (int fd) {
 
 static char addr2str_buf[INET6_ADDRSTRLEN];
 
-static const char *addr2str (const sockaddr_any *addr) {
+const char *addr2str (const sockaddr_any *addr) {
 
 	getnameinfo (&addr->sa, sizeof (*addr),
 		addr2str_buf, sizeof (addr2str_buf), 0, 0, NI_NUMERICHOST);
@@ -1226,8 +1240,6 @@ void tune_socket (int sk) {
 
 
 void parse_icmp_res (probe *pb, int type, int code, int info) {
-	char *str = NULL;
-	char buf[sizeof (pb->err_str)];
 
 	if (af == AF_INET) {
 
@@ -1235,60 +1247,60 @@ void parse_icmp_res (probe *pb, int type, int code, int info) {
 		if (code == ICMP_EXC_TTL)
 			return;
 	    }
-	    else if (type == ICMP_DEST_UNREACH) {
+
+	    if (type == ICMP_DEST_UNREACH) {
 
 		switch (code) {
 		    case ICMP_UNREACH_NET:
 		    case ICMP_UNREACH_NET_UNKNOWN:
 		    case ICMP_UNREACH_ISOLATED:
 		    case ICMP_UNREACH_TOSNET:
-			    str = "!N";
+			    put_err (pb, "!N");
 			    break;
 
 		    case ICMP_UNREACH_HOST:
 		    case ICMP_UNREACH_HOST_UNKNOWN:
 		    case ICMP_UNREACH_TOSHOST:
-			    str = "!H";
+			    put_err (pb, "!H");
 			    break;
 
 		    case ICMP_UNREACH_NET_PROHIB:
 		    case ICMP_UNREACH_HOST_PROHIB:
 		    case ICMP_UNREACH_FILTER_PROHIB:
-			    str = "!X";
+			    put_err (pb, "!X");
 			    break;
 
 		    case ICMP_UNREACH_PORT:
 			    /*  dest host is reached   */
-			    str = "";
 			    break;
 
 		    case ICMP_UNREACH_PROTOCOL:
-			    str = "!P";
+			    put_err (pb, "!P");
 			    break;
 
 		    case ICMP_UNREACH_NEEDFRAG:
-			    snprintf (buf, sizeof (buf), "!F-%d", info);
-			    str = buf;
+			    put_err (pb, "!F-%d", info);
 			    break;
 
 		    case ICMP_UNREACH_SRCFAIL:
-			    str = "!S";
+			    put_err (pb, "!S");
 			    break;
 
 		    case ICMP_UNREACH_HOST_PRECEDENCE:
-			    str = "!V";
+			    put_err (pb, "!V");
 			    break;
 
 		    case ICMP_UNREACH_PRECEDENCE_CUTOFF:
-			    str = "!C";
+			    put_err (pb, "!C");
 			    break;
 
 		    default:
-			    snprintf (buf, sizeof (buf), "!<%u>", code);
-			    str = buf;
+			    put_err (pb, "!<%u>", code);
 			    break;
 		}
-	    }
+
+	    } else
+		put_err (pb, "!<%u-%u>", type, code);
 
 	}
 	else if (af == AF_INET6) {
@@ -1297,49 +1309,37 @@ void parse_icmp_res (probe *pb, int type, int code, int info) {
 		if (code == ICMP6_TIME_EXCEED_TRANSIT)
 			return;
 	    }
-	    else if (type == ICMP6_DST_UNREACH) {
+
+	    if (type == ICMP6_DST_UNREACH) {
 
 		switch (code) {
 
 		    case ICMP6_DST_UNREACH_NOROUTE:
-			    str = "!N";
+			    put_err (pb, "!N");
 			    break;
 
 		    case ICMP6_DST_UNREACH_BEYONDSCOPE:
 		    case ICMP6_DST_UNREACH_ADDR:
-			    str = "!H";
+			    put_err (pb, "!H");
 			    break;
 
 		    case ICMP6_DST_UNREACH_ADMIN:
-			    str = "!X";
+			    put_err (pb, "!X");
 			    break;
 
 		    case ICMP6_DST_UNREACH_NOPORT:
 			    /*  dest host is reached   */
-			    str = "";
 			    break;
 
 		    default:
-			    snprintf (buf, sizeof (buf), "!<%u>", code);
-			    str = buf;
+			    put_err (pb, "!<%u>", code);
 			    break;
 		}
 	    }
-	    else if (type == ICMP6_PACKET_TOO_BIG) {
-		snprintf (buf, sizeof (buf), "!F-%d", info);
-		str = buf;
-	    }
-	}
-
-
-	if (!str) {
-	    snprintf (buf, sizeof (buf), "!<%u-%u>", type, code);
-	    str = buf;
-	}
-
-	if (*str) {
-	    strncpy (pb->err_str, str, sizeof (pb->err_str));
-	    pb->err_str[sizeof (pb->err_str) - 1] = '\0';
+	    else if (type == ICMP6_PACKET_TOO_BIG)
+		    put_err (pb, "!F-%d", info);
+	    else
+		put_err (pb, "!<%u-%u>", type, code);
 	}
 
 	pb->final = 1;
@@ -1351,7 +1351,7 @@ void parse_icmp_res (probe *pb, int type, int code, int info) {
 static void parse_local_res (probe *pb, int ee_errno, int info) {
 
 	if (ee_errno == EMSGSIZE && info != 0) {
-	    snprintf (pb->err_str, sizeof(pb->err_str)-1, "!F-%d", info);
+	    put_err (pb, "!F-%d", info);
 	    pb->final = 1;
 	    return;
 	}
@@ -1523,7 +1523,7 @@ void recv_reply (int sk, int err, check_reply_t check_reply) {
 	      but fill its `err_str' by the info obtained. Ugly, but easy...
 	    */
 	    memset (pb, 0, sizeof (*pb));
-	    snprintf (pb->err_str, sizeof(pb->err_str)-1, "F=%d", ee->ee_info);
+	    put_err (pb, "F=%d", ee->ee_info);
 
 	    return;
 	}
